#!/usr/bin/ruby

if File.exist?('/etc/froxtester.conf')
  load '/etc/froxtester.conf'
elsif File.exist?('froxtester.conf')
  load 'froxtester.conf'
else
  raise "Can't find config file"
end

require 'proxy'
require 'gentests'
require 'cachetests'
require 'acltests'

binaries = []
Dir.foreach(TesterConfig::FroxBinDir) {|file|
  binaries << file if file =~/^frox/
}

errors = Hash.new

tot_errors = 0
tot_tests = 0
tot_retrs = 0
tot_conns = 0

begin
  Dir.mkdir(TesterConfig::WorkingDir + '/wdir') \
                unless File.exist?(TesterConfig::WorkingDir + '/wdir')
  File.new(TesterConfig::WorkingDir + '/wdir/frox.log', 'w').close
  File.truncate(TesterConfig::WorkingDir + '/wdir/frox.log', 0)
rescue
  puts "Error setting up working dir. Check it exists and is writeable"
  exit
end

binaries.each_index {|i|
  puts "Testing #{binaries[i]} - binary #{i+1}/#{binaries.size}"
  frox = Frox.new(TesterConfig::FroxBinDir + "/" + binaries[i],
                  TesterConfig::WorkingDir + "/frox.conf")
  begin
    tester = FroxTester.new(frox)
    tester.extend GeneralTests
    tester.extend GeneralTweaks
    tester.extend AclTests
    tester.extend HttpCacheTests if binaries[i].index("hc")
    tester.extend LocalCacheTests if binaries[i].index("lc")
    errors[binaries[i]], t, c, r = tester.run
    tot_errors += errors[binaries[i]].size
    tot_tests += t
    tot_conns += c
    tot_retrs += r
  ensure
    tester.quit if tester
    frox.kill
  end
}

puts "\nSummary:"
puts "  #{tot_tests} tests run" +
           (tot_errors==0 ? "." : " with #{tot_errors} failures.")
puts "  #{tot_conns} connections made, and #{tot_retrs} files downloaded."
completesuccess=true
errors.each do |key, errorlist|
  completesuccess=false if !errorlist.empty?
  puts "Errors for #{key}:\n" if !errorlist.empty?
  errorlist.each do |err| puts "  #{err}" end
end
puts " All tests passed" if completesuccess
